from copy import deepcopy,copy
from lattice import LOW, HIGH
import pickle

class ddict(dict):
    def __init__(self,default,*args,**kwargs) :
        self.default = default
        dict.__init__(self,*args,**kwargs)

    def __str__(self):
        return 'ddict('+self.default.__repr__()+','+self.__repr__()+')'

    @property
    def show(self):
        output='{'
        for var, level in self.iteritems():
            output+= '\'%s\' -> ' % (var,)
            if level==self.default:
                output+= '\033[1;32m%s\033[1;m ;' % (level,)
            else:
                output+= '\033[1;31m%s\033[1;m ;' % (level,)
        output+='}'
        return output

    def __getitem__(self, key):
        try:
            val = dict.__getitem__(self, key)
        except KeyError:
            val = deepcopy(self.default)
        return val

def kappaDB():
        try:
            kappaDB=pickle.load( open( 'kappa.DB' ) )
        except IOError:
            kappaDB = {}
        return kappaDB

class Kappa():
    def __init__(self,hs):
        self.hash=hs
        self.ssinks=[]
        K=kappaDB()
        try:
            self.map=K[hs]
        except KeyError:
            self.map = {}

    def save(self,hs=None):
        if not hs: hs=self.hash
        K=kappaDB()
        K[hs]=self.map
        pickle.dump( K, open('kappa.DB', "wb" ) )

    def graph(self,delta):
        import re
        dotcode='digraph G {\nsplines="true"\nrankdir=BT\n'
        aftercode=''
        clusters=dict()
        for src, dsts in self.map.iteritems():
            cluster_label=src.split(':',1)
            try:
                cluster=clusters[cluster_label[0]]
            except KeyError:
                cluster='label="'+cluster_label[0]+'";\n'
            cluster+='node [label=\"'+cluster_label[1]+'\"] "'+src+'"'
            if delta[src] == HIGH: cluster+=' [color = red];\n'
            if delta[src] == LOW: cluster+=';\n'
            for dst in dsts:
                aftercode+='\"'+src+'\" -> \"'+dst+ '\"\n'
            clusters[cluster_label[0]]=cluster
        for name, content in clusters.iteritems():
            dotcode+='subgraph cluster_'+re.sub(r'[/\.]','',name)+' {\n'+content
            if self.hash != name: dotcode+='}\n'
        dotcode+=aftercode+'}\n'
        try:
            for sink in self.ssinks:
                dotcode+='"'+sink+'" [color=blue, shape=diamond]\n'
        finally:
            dotcode+='}'
        return dotcode

    def __getitem__(self, key):
        try:
            val = self.map[key]
        except KeyError:
            val = set()
        return val

    def __setitem__(self, key, val):
        self.map[key]=self[key] | val

    def __str__(self):
        return str(self.map)

    def __or__(self,other):
        self.map.update(other.map)
        return self

    @property
    def dom(self):
        return set(self.map.keys())

    def depsIn(self,hash):
        ret=Kappa(hash)
        for src, dsts in self.map.iteritems():
            if src.split(':',1)[0] == hash:
                ret[src]|=dsts
        return ret

def hash(string):
    from hashlib import md5 as algorithm
    from crypt import crypt

    hs=algorithm(string).hexdigest()
    return crypt(hs,'hs')[2:]

